#######################################
##R code for Table 1: compare computation time for quasi-Newton and stratified quasi-Newton######
###########################################


########################################################
##output at the end: time for quasi-Newton and stratified quasi-Newton
#########################################################


#####warning##################
#Newton Method based on extended data fail for large sample size, e.g., N>=5000
#Thus, we save R code for Newton method in a separated file named "Table1_code_Newton"
#############################

#####R code for Table 1######


rm(list=ls())  #remove list

library(survival)
library(mvtnorm)
source("F:/class/JCGS/simulation/revision/function.r")

####With roughly 500 patients for each facility:
##F=2 lead to N=1,000
##F=10 lead to N=5,000
##F=20 lead to N=10,000
##F=200 lead to N=100,000
##########################


F=200 #Number of facility
n_f = rpois(F, lambda = 500)  #sample size for each facility
N=sum(n_f)
p=20
#gamma = exp(rnorm(F, mean=0, sd=0.4))
gamma=rep(1,F)
range(gamma)

K=10
knot=K
tol<-1.0e-9

test_R_all=NULL
test_quasi_all=NULL
test_quasi_all_stratify=NULL



constant_beta_all=NULL
beta.quasi.all=NULL
beta.quasi.all_stratify=NULL

############generate data########################
Sigma_z1=diag(p)
z= rmvnorm(N, mean=rep(0,p), sigma=Sigma_z1)
#z[,4]=rbinom(n=N, size=1, prob=0.5)

U=runif(N, 0,1)
F_pre=1:F
facility=rep(F_pre, n_f)

time=NULL
F_key=0
i_track=0
while (F_key<F){
  F_key=F_key+1
  time_pre=rep(0, n_f[F_key])
  for (i in 1:n_f[F_key]) {
    i_track=i_track+1
    f=function(t) {
      integrand <- function(x) {gamma[F_key]*exp(z[i_track,1]-z[i_track,3]+3*sin(3*pi*x/4)*(x<3)*z[i_track,2]-(x/3)**10*exp(x/4)*(x<3)*z[i_track,4]+sum(z[i_track,5:13])-sum(z[i_track,14:20]))}
      Lambda=integrate(integrand, lower = 0, upper = t)$value
      Lambda+log(1-U[i_track])
    }
    r1 <- suppressWarnings(try(uniroot(f,  lower = 0, upper = 4), silent=TRUE))
    if (class(r1) == "try-error"){    
      time_pre[i]=4
    }
    else time_pre[i]=uniroot(f,  lower = 0, upper = 4)$root
  }
  time=c(time, time_pre)
}

censoring=runif(N,1,30)
censoring=censoring*(censoring<3)+3*(censoring>=3)
tcens=(censoring<time) # censoring indicator
delta=1-tcens
time=time*(delta==1)+censoring*(delta==0)

mean(delta)  # 0.8

delta = delta[order(time)]
z = z[order(time),]
facility=facility[order(time)]
time = time[order(time)]

time2=time[delta==1]
knot_set=quantile(time2,prob=seq(1:(knot-4))/(knot-3))
bs7=bs(time,df=knot, knot=knot_set, intercept=TRUE, degree=3)
bs8=matrix(bs7, nrow=N) # for build beta_t

  
  #############initial values
  
  old_time3=proc.time()
  vfit <- coxph(Surv(time,delta) ~z)
  constant_beta<-vfit$coef
  constant_beta_all=rbind(constant_beta_all, constant_beta) 
  
  loglik_cox <-vfit$loglik[2]
  
  
  temp <- cox.zph(vfit)
  scaled_schoenfeld<-temp$y  ###scaled_schoenfeld+beta_constant
  test_R=temp$table[,3]
  test_R_all=rbind(test_R_all, test_R)
  
  new_time5=proc.time()-old_time3
  new_time5  #N=1000=>0.05; N=30,000=>4.80
  
  
  
  ###quasi rank 2
  old_time3=proc.time()
  theta_quasi=matrix(rep(constant_beta, knot), nrow=p, byrow=FALSE)
  partial_likelihood_quasi=NULL
  key=0
  repeat{
    key=key+1
    if (key==1){
      
      out_quasi=ddloglik_schoenfeld(n=N,delta,z=z,beta=constant_beta,B_spline=bs8, knot=knot)
      H=out_quasi$GVG
      
      g_pre=-matrix(out_quasi$GR_test,ncol=1)
      d_pre=matrix(t(theta_quasi), ncol=1)   
      
      dist=matrix(solve(H)%*%g_pre, nrow=p, byrow=TRUE)
      theta_quasi=theta_quasi-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi)
    
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi), ncol=1)-d_pre
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
      
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi), ncol=1)
      
    }

    if (key>1){

      dist=matrix(solve(H)%*%matrix(out_quasi$L1,ncol=1), nrow=p, byrow=TRUE)
      theta_quasi=theta_quasi-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi)
      
      
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi), ncol=1)-d_pre
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi), ncol=1)
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
    }
    #partial_likelihood_quasi=rbind(partial_likelihood_quasi, out_quasi$partial_likelihood)
    
    if(max(abs(dist))<tol) break
  }
  
  
  new_time_quasi=proc.time()-old_time3

  

  beta.quasi.all=rbind(beta.quasi.all, theta_quasi)
  test_quasi=rep(0,p)
constrain=-diff(diag(knot*1),differences=1)
  j=0
  repeat{
    j=j+1
    
    theta_quasi_j= theta_quasi[j,]
    
    L2=solve(H)[((j-1)*knot+1):((j-1)*knot+knot),((j-1)*knot+1):((j-1)*knot+knot)]
    test_contrast=t(constrain%*%theta_quasi_j)%*%solve(constrain%*%L2%*%t(constrain))%*%(constrain%*%theta_quasi_j)
    test_quasi[j]=1-pchisq(test_contrast, (knot-1))
    if(j==p) break
  }
  
  test_quasi_all=rbind(test_quasi_all, test_quasi)
  
  
  
  ###stratified quasi rank 2
  old_time3=proc.time()
  theta_quasi_stratify=matrix(rep(constant_beta, knot), nrow=p, byrow=FALSE)
  partial_likelihood_quasi_stratify=NULL
  key=0
  repeat{
    key=key+1
    if (key==1){
      
      out_quasi=ddloglik_schoenfeld_stratify(n=N,delta,z=z,beta=constant_beta,B_spline=bs8, knot=knot, facility=facility, number_facility=F)
      H=out_quasi$GVG
      
      g_pre=-matrix(out_quasi$GR_test,ncol=1)
      d_pre=matrix(t(theta_quasi_stratify), ncol=1)   
      
      dist=matrix(solve(H)%*%g_pre, nrow=p, byrow=TRUE)
      theta_quasi_stratify=theta_quasi_stratify-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1_stratify(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi_stratify, facility=facility, number_facility=F)
      
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi_stratify), ncol=1)-d_pre
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
      
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi_stratify), ncol=1)
      
    }
    
    if (key>1){
      
      dist=matrix(solve(H)%*%matrix(out_quasi$L1,ncol=1), nrow=p, byrow=TRUE)
      theta_quasi_stratify=theta_quasi_stratify-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1_stratify(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi_stratify, facility=facility, number_facility=F)
      
      
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi_stratify), ncol=1)-d_pre
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi_stratify), ncol=1)
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
    }
    #partial_likelihood_quasi=rbind(partial_likelihood_quasi, out_quasi$partial_likelihood)
    
    if(max(abs(dist))<tol) break
  }
  new_time_statified_quasi=proc.time()-old_time3
  

  beta.quasi.all_stratify=rbind(beta.quasi.all_stratify, theta_quasi_stratify)
  test_quasi_stratify=rep(0,p)
  j=0
  repeat{
    j=j+1
    
    theta_quasi_j= theta_quasi_stratify[j,]
    
    L2=solve(H)[((j-1)*knot+1):((j-1)*knot+knot),((j-1)*knot+1):((j-1)*knot+knot)]
    test_contrast=t(constrain%*%theta_quasi_j)%*%solve(constrain%*%L2%*%t(constrain))%*%(constrain%*%theta_quasi_j)
    test_quasi_stratify[j]=1-pchisq(test_contrast, (knot-1))
    if(j==p) break
  }
  
  test_quasi_all_stratify=rbind(test_quasi_all_stratify, test_quasi_stratify)
  
  
  
new_time_quasi 
##N=1000  39.22 
##N=5000  344.39  
##N=10000  1487.64 
new_time_statified_quasi 
##N=1000  27.02
###N=5000 51.67
##N=10000  93.39
##N=100000  1107.59 










####stratified Newton

old_time3=proc.time()
delta_new=NULL
time_new=NULL
z_new=NULL
bs_new=NULL
time_start=NULL
time_stop=NULL
facility_new=NULL

###########extend data for time-varying###############
for (i in 1:F) {
  data_new<-data_extend_stratify(delta[facility==i], time[facility==i], z[facility==i,], bs8[facility==i,], facility=facility[facility==i])  ###a function written in function.R
  index=which(data_new$time_start==data_new$time_stop)
  delta_new<-c(delta_new,data_new$delta_new[-index])
  time_new<-c(time_new, data_new$time_new[-index])
  z_new<-rbind(z_new,data_new$z_new[-index,])
  bs_new<-rbind(bs_new,data_new$bs_new[-index,])
  time_start<-c(time_start,data_new$time_start[-index])
  time_stop<-c(time_stop, data_new$time_stop[-index])
  facility_new=c(facility_new, data_new$facility_new[-index])
}
###########################################################################
#time_stop-time_start

#data_new$time_start[data_new$time_start==data_new$time_stop]
#sum(data_new$time_start==data_new$time_stop)
#mean(delta[facility==i])
#sum(facility==i)
#length(data_new$time_start)

theta=coxph(Surv(time_start, time_stop, delta_new)~z_new:bs_new+strata(facility_new))$coef

theta2=matrix(theta,nrow=p)

new_time_Newton_stratify=proc.time()-old_time3
new_time_Newton_stratify  
#N=1000; 179.50
#N=5000; 717.55 
#N=10000; Fail


####unstratified Newton



old_time3=proc.time()

###########extend data for time-varying###############
data_new<-data_extend(delta, time, z, bs8)  ###a function written in function.R
index=which(data_new$time_start==data_new$time_stop)
delta_new<-data_new$delta_new[-index]
time_new<-data_new$time_new[-index]
z_new<-data_new$z_new[-index,]
bs_new<-data_new$bs_new[-index,]
time_start<-data_new$time_start[-index]
time_stop<-data_new$time_stop[-index]


###########################################################################
#sum(time_start==time_stop)
theta=coxph(Surv(time_start, time_stop, delta_new)~z_new:bs_new)$coef

theta2=matrix(theta,nrow=p)

new_time_Newton=proc.time()-old_time3
new_time_Newton  
#N=1000; 422.80 seconds
#N=5000; Fail 











###############################################
##R code for simulation figures 1-2##
##10 simulation loops (take 1-2 minutes) to illustrate the code; manuscript used 100 simulation loops
#######################################################33




rm(list=ls())  #remove list

library(survival)
library(mvtnorm)

source("F:/class/JCGS/simulation/revision/function.r")

########################################

magnitude=2  ####control magnitude for time-varying effects
#### magnitude=3  for Figure2
#### magnitude=2 is for Figure1
#######################################


F=5 #Number of facility
n_f = rpois(F, lambda = 200)  #sample size for each facility
N=sum(n_f)
p=3
#gamma = exp(rnorm(F, mean=0, sd=0.4))
#Mean=rnorm(F, mean=0, sd=0.4)
gamma=rep(1,F)
Mean=rep(0,F)
range(gamma)

K=10
knot=K
tol<-1.0e-9

test_R_all=NULL
beta.NR.all=NULL

test_quasi_all=NULL

constant_beta_all=NULL
beta.quasi.all=NULL
schoenfeld_time_all=NULL
quasi_time_all=NULL
one_step_Newton_time_all=NULL
quasi_time_all=NULL
quasi_key=NULL

quasi_statify_key=NULL 
quasi_stratify_time_all=NULL
beta.quasi.all_stratify=NULL
test_quasi_all_stratify=NULL
Lambda.quasi.all=NULL
Lambda.quasi.stratify.all=NULL
time_all=NULL
beta.quasi.constrain.all=NULL
beta.quasi.stratify.constrain.all=NULL
H_all=NULL
H_stratify_all=NULL

H.constrain_all=NULL
H.constrain_stratify_all=NULL

#max_ite=20

nloop=500   ###10 simulation loops to illustrate the code; manuscript used 100 simulation loops###

all_time=proc.time()

loop=0

repeat {
  loop=loop+1
  
  set.seed(loop)
  
  ############generate data########################
  Sigma_z1=diag(p)
  z= matrix(0, nrow=N, ncol=p)
  
  
  U=runif(N, 0,1)
  F_pre=1:F
  facility=rep(F_pre, n_f)
  
  time=NULL
  F_key=0
  i_track=0
  while (F_key<F){
    F_key=F_key+1
    z[facility==F_key,]=rmvnorm(sum(facility==F_key), mean=rep(Mean[F_key],p), sigma=Sigma_z1)
    
    #gamma = exp((F_key-floor(F/2)-1)/F)
    
    time_pre=rep(0, n_f[F_key])
    for (i in 1:n_f[F_key]) {
      i_track=i_track+1
      f=function(t) {
        integrand <- function(x) {gamma[F_key]*exp(z[i_track,1]-z[i_track,3]+3*sin(3*pi*x/4)*(x<3)*z[i_track,2])}
        Lambda=integrate(integrand, lower = 0, upper = t)$value
        Lambda+log(1-U[i_track])
      }
      r1 <- suppressWarnings(try(uniroot(f,  lower = 0, upper = 4), silent=TRUE))
      if (class(r1) == "try-error"){    
        time_pre[i]=4
      }
      else time_pre[i]=uniroot(f,  lower = 0, upper = 4)$root
    }
    time=c(time, time_pre)
  }
  
  censoring=runif(N,1,30)
  censoring=censoring*(censoring<3)+3*(censoring>=3)
  tcens=(censoring<time) # censoring indicator
  delta=1-tcens
  time=time*(delta==1)+censoring*(delta==0)
  
  mean(delta)  # 0.8
  
  delta = delta[order(time)]
  z = z[order(time),]
  facility=facility[order(time)]
  time = time[order(time)]
  
  
  time2=time[delta==1]
  knot_set=quantile(time2,prob=seq(1:(knot-4))/(knot-3))
  bs7=bs(time,df=knot, knot=knot_set, intercept=TRUE, degree=3)
  bs8=matrix(bs7, nrow=N) # for build beta_t
  
  
  
  
  schoenfeld_time=proc.time()
  vfit <- coxph(Surv(time,delta) ~z)
  constant_beta<-vfit$coef
  constant_beta_all=rbind(constant_beta_all, constant_beta) 
  
  loglik_cox <-vfit$loglik[2]
  
  
  temp <- cox.zph(vfit)
  scaled_schoenfeld<-temp$y  ###scaled_schoenfeld+beta_constant
  test_R=temp$table[,3]
  test_R_all=rbind(test_R_all, test_R)
  
  schoenfeld_time=proc.time()-schoenfeld_time
  schoenfeld_time_all=c(schoenfeld_time_all, schoenfeld_time[3])  
  
  
  #################One-step Newton method
  one_step_Newton_time=proc.time()
  theta_NR=matrix(rep(constant_beta, knot), nrow=p, byrow=FALSE)
  
  temp=ddloglik_schoenfeld_bs_t(n=N,delta,z=z,beta=constant_beta,B_spline=bs8, knot=knot, theta=theta_NR)
  dist=matrix(solve(temp$GVG)%*%matrix(temp$GR_test,ncol=1), nrow=p, byrow=TRUE)
  theta_NR=theta_NR+dist
  
  one_step_Newton_time=proc.time()-one_step_Newton_time
  one_step_Newton_time_all=c(one_step_Newton_time_all,one_step_Newton_time[3])  #N=10000 1646.32
  beta.NR.all=rbind(beta.NR.all, theta_NR)
  #################################
  
  constrain=-diff(diag(knot*1),differences=1)
  
  
  
  
  ###quasi rank 2
  quasi_time=proc.time()
  theta_quasi=matrix(rep(constant_beta, knot), nrow=p, byrow=FALSE)
  partial_likelihood_quasi=NULL
  key=0
  repeat{
    key=key+1
    if (key==1){
      
      out_quasi=ddloglik_schoenfeld(n=N,delta,z=z,beta=constant_beta,B_spline=bs8, knot=knot)
      H=out_quasi$GVG
      
      g_pre=-matrix(out_quasi$GR_test,ncol=1)
      d_pre=matrix(t(theta_quasi), ncol=1)   
      
      dist=matrix(solve(H)%*%g_pre, nrow=p, byrow=TRUE)
      theta_quasi=theta_quasi-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi)
      
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi), ncol=1)-d_pre
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
      
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi), ncol=1)
      
    }
    
    if (key>1){
      
      dist=matrix(solve(H)%*%matrix(out_quasi$L1,ncol=1), nrow=p, byrow=TRUE)
      theta_quasi=theta_quasi-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi)
      
      
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi), ncol=1)-d_pre
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi), ncol=1)
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
    }
    #partial_likelihood_quasi=rbind(partial_likelihood_quasi, out_quasi$partial_likelihood)
    
    if(max(abs(dist))<tol) break
  }
  
  quasi_key=c(quasi_key,key) 
  
  quasi_time=proc.time()-quasi_time
  quasi_time_all=c(quasi_time_all,quasi_time[3])  
  
  beta.quasi.all=rbind(beta.quasi.all, theta_quasi)
  
  
  C_lambda= Lambda(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi)$Lambda
  Lambda.quasi.all=rbind(Lambda.quasi.all, C_lambda)
  time_all=rbind(time_all,time)
  
  H_all[[loop]]=H
  
  
  test_quasi=rep(0,p)
  j=0
  repeat{
    j=j+1
    
    theta_quasi_j= theta_quasi[j,]
    
    L2=solve(H)[((j-1)*knot+1):((j-1)*knot+knot),((j-1)*knot+1):((j-1)*knot+knot)]
    test_contrast=t(constrain%*%theta_quasi_j)%*%solve(constrain%*%L2%*%t(constrain))%*%(constrain%*%theta_quasi_j)
    test_quasi[j]=1-pchisq(test_contrast, (knot-1))
    if(j==p) break
  }
  
  test_quasi_all=rbind(test_quasi_all, test_quasi)
  
  independent=which(test_quasi>0.05)
  
  if (length(independent)==0){
    beta.quasi.constrain.all=rbind(beta.quasi.constrain.all, theta_quasi)
    H_all[[loop]]=H
  } else {
    constrain_matrix=NULL
    for (j in 1:p){
      if (j %in% independent) {
        constrain_temp=matrix(0,nrow=knot-1,ncol=p*knot)
        constrain_temp[, ((j-1)*knot+1):(j*knot)]=-diff(diag(knot*1),differences=1)
        constrain_matrix=rbind(constrain_matrix,constrain_temp)
      } 
    }
    
    ###quasi rank 2 with constrain
    theta_quasi=matrix(rep(constant_beta, knot), nrow=p, byrow=FALSE)
    partial_likelihood_quasi=NULL
    key=0
    repeat{
      key=key+1
      if (key==1){
        
        out_quasi=ddloglik_schoenfeld(n=N,delta,z=z,beta=constant_beta,B_spline=bs8, knot=knot)
        H=out_quasi$GVG
        
        g_pre=-matrix(out_quasi$GR_test,ncol=1)
        d_pre=matrix(t(theta_quasi), ncol=1)   
        
        dist=matrix((solve(H)-solve(H)%*%t(constrain_matrix)%*%solve(constrain_matrix%*%solve(H)%*%t(constrain_matrix))%*%constrain_matrix%*%solve(H))%*%g_pre, nrow=p, byrow=TRUE)
        theta_quasi=theta_quasi-dist
        
        out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi)
        
        g=matrix(out_quasi$L1,ncol=1)-g_pre
        d=matrix(t(theta_quasi), ncol=1)-d_pre
        b=as.vector(1/(t(g)%*%d))
        c=as.vector(-1/(t(d)%*%H%*%d))
        H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
        
        g_pre=matrix(out_quasi$L1,ncol=1)
        d_pre=matrix(t(theta_quasi), ncol=1)
        
      }
      
      if (key>1){
        
        dist=matrix((solve(H)-solve(H)%*%t(constrain_matrix)%*%solve(constrain_matrix%*%solve(H)%*%t(constrain_matrix))%*%constrain_matrix%*%solve(H))%*%out_quasi$L1, nrow=p, byrow=TRUE)
        theta_quasi=theta_quasi-dist
        
        out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi)
        
        
        g=matrix(out_quasi$L1,ncol=1)-g_pre
        d=matrix(t(theta_quasi), ncol=1)-d_pre
        g_pre=matrix(out_quasi$L1,ncol=1)
        d_pre=matrix(t(theta_quasi), ncol=1)
        b=as.vector(1/(t(g)%*%d))
        c=as.vector(-1/(t(d)%*%H%*%d))
        H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
      }
      #partial_likelihood_quasi=rbind(partial_likelihood_quasi, out_quasi$partial_likelihood)
      
      if(max(abs(dist))<tol) break
    }
    
    beta.quasi.constrain.all=rbind(beta.quasi.constrain.all, theta_quasi)
    H.constrain_all[[loop]]=H
  }
  
  
  ###stratified quasi rank 2
  quasi_stratify_time=proc.time()
  theta_quasi_stratify=matrix(rep(constant_beta, knot), nrow=p, byrow=FALSE)
  partial_likelihood_quasi_stratify=NULL
  key=0
  repeat{
    key=key+1
    if (key==1){
      
      out_quasi=ddloglik_schoenfeld_stratify(n=N,delta,z=z,beta=constant_beta,B_spline=bs8, knot=knot, facility=facility, number_facility=F)
      H=out_quasi$GVG
      
      g_pre=-matrix(out_quasi$GR_test,ncol=1)
      d_pre=matrix(t(theta_quasi_stratify), ncol=1)   
      
      dist=matrix(solve(H)%*%g_pre, nrow=p, byrow=TRUE)
      theta_quasi_stratify=theta_quasi_stratify-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1_stratify(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi_stratify, facility=facility, number_facility=F)
      
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi_stratify), ncol=1)-d_pre
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
      
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi_stratify), ncol=1)
      
    }
    
    if (key>1){
      
      dist=matrix(solve(H)%*%matrix(out_quasi$L1,ncol=1), nrow=p, byrow=TRUE)
      theta_quasi_stratify=theta_quasi_stratify-dist
      
      out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1_stratify(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi_stratify, facility=facility, number_facility=F)
      
      
      g=matrix(out_quasi$L1,ncol=1)-g_pre
      d=matrix(t(theta_quasi_stratify), ncol=1)-d_pre
      g_pre=matrix(out_quasi$L1,ncol=1)
      d_pre=matrix(t(theta_quasi_stratify), ncol=1)
      b=as.vector(1/(t(g)%*%d))
      c=as.vector(-1/(t(d)%*%H%*%d))
      H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
    }
    #partial_likelihood_quasi=rbind(partial_likelihood_quasi, out_quasi$partial_likelihood)
    
    if(max(abs(dist))<tol) break
  }
  
  quasi_statify_key=c(quasi_statify_key,key)  #N=1000=>5.82; N=100,000=>13.77 hour
  
  ###N=1000 this code time=5.82; Krnoencker product: 46.62 seconds 
  quasi_stratify_time=proc.time()-quasi_stratify_time
  quasi_stratify_time_all=c(quasi_stratify_time_all,quasi_stratify_time[3])  #N=1000=>5.82; N=100,000=>13.77 hour
  
  
  beta.quasi.all_stratify=rbind(beta.quasi.all_stratify, theta_quasi_stratify)
  C_lambda_stratify= Lambda_stratify(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi_stratify, facility=facility, number_facility=F)$Lambda
  Lambda.quasi.stratify.all=rbind(Lambda.quasi.stratify.all, C_lambda_stratify)
  
  H_stratify_all[[loop]]=H
  
  test_quasi_stratify=rep(0,p)
  j=0
  repeat{
    j=j+1
    
    theta_quasi_j= theta_quasi_stratify[j,]
    
    L2=solve(H)[((j-1)*knot+1):((j-1)*knot+knot),((j-1)*knot+1):((j-1)*knot+knot)]
    test_contrast=t(constrain%*%theta_quasi_j)%*%solve(constrain%*%L2%*%t(constrain))%*%(constrain%*%theta_quasi_j)
    test_quasi_stratify[j]=1-pchisq(test_contrast, (knot-1))
    if(j==p) break
  }
  
  test_quasi_all_stratify=rbind(test_quasi_all_stratify, test_quasi_stratify)
  
  ###stratified quasi rank 2 with constraint
  
  independent=which(test_quasi_stratify>0.05)
  
  if (length(independent)==0){
    beta.quasi.stratify.constrain.all=rbind(beta.quasi.stratify.constrain.all, theta_quasi_stratify)
    H_stratify_all[[loop]]=H
    } else {
    constrain_matrix=NULL
    for (j in 1:p){
      if (j %in% independent) {
        constrain_temp=matrix(0,nrow=knot-1,ncol=p*knot)
        constrain_temp[, ((j-1)*knot+1):(j*knot)]=-diff(diag(knot*1),differences=1)
        constrain_matrix=rbind(constrain_matrix,constrain_temp)
      } 
    }
    
    
    theta_quasi_stratify=matrix(rep(constant_beta, knot), nrow=p, byrow=FALSE)
    partial_likelihood_quasi_stratify=NULL
    key=0
    repeat{
      key=key+1
      if (key==1){
        
        out_quasi=ddloglik_schoenfeld_stratify(n=N,delta,z=z,beta=constant_beta,B_spline=bs8, knot=knot, facility=facility, number_facility=F)
        H=out_quasi$GVG
        
        g_pre=-matrix(out_quasi$GR_test,ncol=1)
        d_pre=matrix(t(theta_quasi_stratify), ncol=1)   
        
        dist=matrix((solve(H)-solve(H)%*%t(constrain_matrix)%*%solve(constrain_matrix%*%solve(H)%*%t(constrain_matrix))%*%constrain_matrix%*%solve(H))%*%g_pre, nrow=p, byrow=TRUE)
        theta_quasi_stratify=theta_quasi_stratify-dist
        
        theta_quasi_stratify=theta_quasi_stratify-dist
        
        out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1_stratify(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi_stratify, facility=facility, number_facility=F)
        
        g=matrix(out_quasi$L1,ncol=1)-g_pre
        d=matrix(t(theta_quasi_stratify), ncol=1)-d_pre
        b=as.vector(1/(t(g)%*%d))
        c=as.vector(-1/(t(d)%*%H%*%d))
        H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
        
        g_pre=matrix(out_quasi$L1,ncol=1)
        d_pre=matrix(t(theta_quasi_stratify), ncol=1)
        
      }
      
      if (key>1){
        
        dist=matrix((solve(H)-solve(H)%*%t(constrain_matrix)%*%solve(constrain_matrix%*%solve(H)%*%t(constrain_matrix))%*%constrain_matrix%*%solve(H))%*%out_quasi$L1, nrow=p, byrow=TRUE)
        
        theta_quasi_stratify=theta_quasi_stratify-dist
        
        out_quasi=ddloglik_schoenfeld_bs_t_quasi_L1_stratify(n=N,delta=delta,z=z,B_spline=bs8,  theta=theta_quasi_stratify, facility=facility, number_facility=F)
        
        
        g=matrix(out_quasi$L1,ncol=1)-g_pre
        d=matrix(t(theta_quasi_stratify), ncol=1)-d_pre
        g_pre=matrix(out_quasi$L1,ncol=1)
        d_pre=matrix(t(theta_quasi_stratify), ncol=1)
        b=as.vector(1/(t(g)%*%d))
        c=as.vector(-1/(t(d)%*%H%*%d))
        H=H+b*g%*%t(g)+c*H%*%d%*%t(d)%*%H
      }
      #partial_likelihood_quasi=rbind(partial_likelihood_quasi, out_quasi$partial_likelihood)
      
      if(max(abs(dist))<tol) break
    }
    
    
    beta.quasi.stratify.constrain.all=rbind(beta.quasi.stratify.constrain.all, theta_quasi_stratify)
    H.constrain_stratify_all[[loop]]=H
    }
  
  if(loop==nloop) break
}

all_time=proc.time()-all_time
all_time  #13571.71



#colMeans(z)

as.vector(sapply(split(z[,1],factor(facility)),mean)) 
as.vector(sapply(split(z[,2],factor(facility)),mean)) 
as.vector(sapply(split(z[,3],factor(facility)),mean)) 


mean(delta)
quantile(time[delta==1])
#quantile(time)
#0%        25%        50%        75%       100% 
#0.00084193 0.15945601 0.54742134 1.52049652 3.00000000 
# quantile(time[delta==1])
#0%        25%        50%        75%       100% 
#0.00084193 0.13309557 0.41464896 1.37081355 2.98683337 



colMeans(test_R_all)
#z1           z2           z3       GLOBAL 
#0.0663647817 0.0006068553 0.0622433665 0.0012706607
colMeans(test_quasi_all) #0.4691803 0.0000000 0.4456681
colMeans(test_quasi_all_stratify) 
#0.4620845 0.0000000 0.4446106
colMeans(test_R_all<0.05) #  0.746  0.996  0.734  0.992 
colMeans(test_quasi_all<0.05) # 0.078 1.000 0.080
colMeans(test_quasi_all_stratify<0.05) #0.072 1.000 0.062


mean(one_step_Newton_time_all) #1.38522
mean(quasi_time_all)#8.93718
mean(quasi_key) #18.512
mean(quasi_statify_key)#19.158
mean(quasi_stratify_time_all)#4.79778



library(splines)

time_MSE=time  #3*(1:100)/100
bs_MSE=bs(time_MSE,df=knot, knot=knot_set, intercept=TRUE, degree=3)
bs_MSE=matrix(bs_MSE, nrow=N) # for build beta_t


beta_true_MSE=matrix( rep(0, length(time_MSE)*p), nrow=length(time_MSE))
beta_true_MSE[,1]=1
beta_true_MSE[,2]=3*sin(3*time_MSE*pi/4)
beta_true_MSE[,3]=-1

MSE_quasi=matrix(rep(0, p*nloop), nrow=nloop)
MSE_quasi_stratify=matrix(rep(0, p*nloop), nrow=nloop)

bias_quasi=matrix(rep(0, p*nloop), nrow=nloop)
bias_quasi_stratify=matrix(rep(0, p*nloop), nrow=nloop)

CP_quasi=NULL
CP_quasi_stratify=NULL

for (loop in 1:nloop) {
  beta_MSE_quasi=matrix( rep(0, length(time_MSE)*p), nrow=length(time_MSE))
  beta_MSE_quasi_stratify=matrix( rep(0, length(time_MSE)*p), nrow=length(time_MSE))
  
  beta_low_quasi=matrix( rep(0, length(time_MSE)*p), nrow=length(time_MSE))
  beta_up_quasi=matrix( rep(0, length(time_MSE)*p), nrow=length(time_MSE))
  
  beta_low_quasi_stratify=matrix( rep(0, length(time_MSE)*p), nrow=length(time_MSE))
  beta_up_quasi_stratify=matrix( rep(0, length(time_MSE)*p), nrow=length(time_MSE))
  
  if (length(H.constrain_all[[loop]])==0){  
    beta_MSE_quasi[,j]=bs_MSE%*%beta.quasi.constrain.all[p*(loop-1)+j,]
    for (j in 1:p) {  
      MSE_quasi[loop,j]=sum((beta_MSE_quasi[,j]-beta_true_MSE[,j])**2)/length(time_MSE)
      bias_quasi[loop,j]=sum(abs(beta_MSE_quasi[,j]-beta_true_MSE[,j]))/length(time_MSE)
    }
    CP_quasi=CP_quasi
  } else {
    H=solve(H.constrain_all[[loop]]) 
    
    for (j in 1:p) {  
      beta_MSE_quasi[,j]=bs_MSE%*%beta.quasi.constrain.all[p*(loop-1)+j,]
      
      MSE_quasi[loop,j]=sum((beta_MSE_quasi[,j]-beta_true_MSE[,j])**2)/length(time_MSE)
      bias_quasi[loop,j]=sum(abs(beta_MSE_quasi[,j]-beta_true_MSE[,j]))/length(time_MSE)
      
      var_quasi=diag(bs_MSE%*%H[(K*(j-1)+1):(K*(j-1)+K), (K*(j-1)+1):(K*(j-1)+K)]%*%t(bs_MSE))
      beta_low_quasi[,j]=beta_MSE_quasi[,j]-1.9645*sqrt(var_quasi)
      beta_up_quasi[,j]=beta_MSE_quasi[,j]+1.9645*sqrt(var_quasi)
      CP_quasi=c(CP_quasi,sum((beta_true_MSE[,j]>beta_low_quasi[,j])*(beta_true_MSE[,j]<beta_up_quasi[,j])))      
    }
  }
  
  if (length(H.constrain_stratify_all[[loop]])==0){   
    for (j in 1:p) {  
      beta_MSE_quasi_stratify[,j]=bs_MSE%*%beta.quasi.stratify.constrain.all[p*(loop-1)+j,]
      MSE_quasi_stratify[loop,j]=sum((beta_MSE_quasi_stratify[,j]-beta_true_MSE[,j])**2)/length(time_MSE)
      bias_quasi_stratify[loop,j]=sum(abs(beta_MSE_quasi_stratify[,j]-beta_true_MSE[,j]))/length(time_MSE)
    }
    CP_quasi_stratify=CP_quasi_stratify
  }else {
    H_stratify=solve(H.constrain_stratify_all[[loop]])
    for (j in 1:p) {  
      beta_MSE_quasi_stratify[,j]=bs_MSE%*%beta.quasi.stratify.constrain.all[p*(loop-1)+j,]
      MSE_quasi_stratify[loop,j]=sum((beta_MSE_quasi_stratify[,j]-beta_true_MSE[,j])**2)/length(time_MSE)
      bias_quasi_stratify[loop,j]=sum(abs(beta_MSE_quasi_stratify[,j]-beta_true_MSE[,j]))/length(time_MSE)
      
      var_quasi_stratify=diag(bs_MSE%*%H_stratify[(K*(j-1)+1):(K*(j-1)+K), (K*(j-1)+1):(K*(j-1)+K)]%*%t(bs_MSE))      
      beta_low_quasi_stratify[,j]=beta_MSE_quasi_stratify[,j]-1.9645*sqrt(var_quasi_stratify)
      beta_up_quasi_stratify[,j]=beta_MSE_quasi_stratify[,j]+1.9645*sqrt(var_quasi_stratify)
      CP_quasi_stratify=c(CP_quasi_stratify,sum((beta_true_MSE[,j]>beta_low_quasi_stratify[,j])*(beta_true_MSE[,j]<beta_up_quasi_stratify[,j])))
    }
  }
}
ISE_quasi=sum(MSE_quasi)/(nloop*p)
ISE_quasi_stratify=sum(MSE_quasi_stratify)/(nloop*p)

ISE_quasi  #0.1319452
ISE_quasi_stratify  #0.1303449

bias_quasi=sum(bias_quasi)/(nloop*p)
bias_quasi_stratify=sum(bias_quasi_stratify)/(nloop*p)

bias_quasi  # 0.1297253
bias_quasi_stratify  # 0.1243647

mean(CP_quasi)/length(time_MSE) #0.9510929
mean(CP_quasi_stratify)/length(time_MSE) #0.9538385






Lambda1=NULL
Lambda2=NULL
Lambda3=NULL

Lambda.stratify1=NULL
Lambda.stratify2=NULL
Lambda.stratify3=NULL

for (i in 1:nloop) {
  Lambda.temp=Lambda.quasi.all[i,]
  time.temp=time_all[i,]
  Lambda1=c(Lambda1,Lambda.temp[sum(time.temp<=0.5)])
  Lambda2=c(Lambda2, Lambda.temp[sum(time.temp<=1)])
  Lambda3=c(Lambda3, Lambda.temp[sum(time.temp<=1.5)])
  
  
  n_track=0
  for (F_key in 1:F) {
    Lambda.stratify.temp=Lambda.quasi.stratify.all[i,(n_track+1):(n_track+n_f[F_key])]
    time.stratify.temp=time.temp[(n_track+1):(n_track+n_f[F_key])]
    Lambda.stratify1=c(Lambda.stratify1, Lambda.stratify.temp[sum(time.stratify.temp<=0.5)])
    Lambda.stratify2=c(Lambda.stratify2, Lambda.stratify.temp[sum(time.stratify.temp<=1)])
    Lambda.stratify3=c(Lambda.stratify3, Lambda.stratify.temp[sum(time.stratify.temp<=1.5)])
    n_track=n_track+n_f[F_key]
  }
}

mean(abs(Lambda1-0.5))  #0.02809971
mean(abs(Lambda2-1))   #0.05100362
mean(abs(Lambda3-1.5))   #0.08026255

mean(abs(Lambda.stratify1-0.5))/F  #0.03932226
mean(abs(Lambda.stratify2-1))/F   #0.1073271
mean(abs(Lambda.stratify3-1.5))/F  #0.1531961


############### quasi Newton  ######
par(mai=c(1.1,1.1,0.5,0.4))
beta_all_2=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_2[,i]=bs8%*%(as.matrix(beta.quasi.all[p*(i-1)+2,]))
}

beta_all_mean=rowMeans(beta_all_2)   
#beta_all_median=apply(beta_all_2, 1, median)

plot(time,beta_all_mean,  ylim=c(-6,5), xlab="Time", ylab=expression(beta[1](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
#plot(time,beta_all_median,  ylim=c(-2,2), xlab="Time", ylab=expression(theta[1](t)), lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, 3*sin(3*pi*time/4), lty=2, col="red")
#abline(0,0, lty=6)

beta_low=rep(0, nrow(beta_all_2))
beta_up=rep(0, nrow(beta_all_2))         
for (i in 1:nrow(beta_all_2))
{
  beta_low[i]=quantile(beta_all_2[i,], probs = 0.025)
  beta_up[i]=quantile(beta_all_2[i,], probs = 0.975)
}
lines(time, beta_low, lty=5)
lines(time, beta_up, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)

##############################################################

beta_all_1=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_1[,i]=bs8%*%(as.matrix(beta.quasi.all[p*(i-1)+1,]))
}

beta_all_mean1=rowMeans(beta_all_1)   

plot(time,beta_all_mean1,  ylim=c(-2,4), xlab="Time", ylab=expression(beta[2](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, rep(1,N), lty=2, col="red")

beta_low1=rep(0, nrow(beta_all_1))
beta_up1=rep(0, nrow(beta_all_1))         
for (i in 1:nrow(beta_all_1))
{
  beta_low1[i]=quantile(beta_all_1[i,], probs = 0.025)
  beta_up1[i]=quantile(beta_all_1[i,], probs = 0.975)
}
lines(time, beta_low1, lty=5)
lines(time, beta_up1, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)





beta_all_1=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_1[,i]=bs8%*%(as.matrix(beta.quasi.all[p*(i-1)+3,]))
}

beta_all_mean1=rowMeans(beta_all_1)   

plot(time,beta_all_mean1,  ylim=c(-4,2), xlab="Time", ylab=expression(beta[3](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, rep(-1,N), lty=2, col="red")

beta_low1=rep(0, nrow(beta_all_1))
beta_up1=rep(0, nrow(beta_all_1))         
for (i in 1:nrow(beta_all_1))
{
  beta_low1[i]=quantile(beta_all_1[i,], probs = 0.025)
  beta_up1[i]=quantile(beta_all_1[i,], probs = 0.975)
}
lines(time, beta_low1, lty=5)
lines(time, beta_up1, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)









############### stratified quasi Newton  ######
par(mai=c(1.1,1.1,0.5,0.4))
beta_all_2=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_2[,i]=bs8%*%(as.matrix(beta.quasi.all_stratify[p*(i-1)+2,]))
}

beta_all_mean=rowMeans(beta_all_2)   
#beta_all_median=apply(beta_all_2, 1, median)

plot(time,beta_all_mean,  ylim=c(-6,5), xlab="Time", ylab=expression(beta[1](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
#plot(time,beta_all_median,  ylim=c(-2,2), xlab="Time", ylab=expression(theta[1](t)), lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, 3*sin(3*pi*time/4), lty=2, col="red")
#abline(0,0, lty=6)

beta_low=rep(0, nrow(beta_all_2))
beta_up=rep(0, nrow(beta_all_2))         
for (i in 1:nrow(beta_all_2))
{
  beta_low[i]=quantile(beta_all_2[i,], probs = 0.025)
  beta_up[i]=quantile(beta_all_2[i,], probs = 0.975)
}
lines(time, beta_low, lty=5)
lines(time, beta_up, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)

##############################################################

beta_all_1=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_1[,i]=bs8%*%(as.matrix(beta.quasi.all_stratify[p*(i-1)+1,]))
}

beta_all_mean1=rowMeans(beta_all_1)   

plot(time,beta_all_mean1,  ylim=c(-2,4), xlab="Time", ylab=expression(beta[2](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, rep(1,N), lty=2, col="red")

beta_low1=rep(0, nrow(beta_all_1))
beta_up1=rep(0, nrow(beta_all_1))         
for (i in 1:nrow(beta_all_1))
{
  beta_low1[i]=quantile(beta_all_1[i,], probs = 0.025)
  beta_up1[i]=quantile(beta_all_1[i,], probs = 0.975)
}
lines(time, beta_low1, lty=5)
lines(time, beta_up1, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)





beta_all_1=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_1[,i]=bs8%*%(as.matrix(beta.quasi.all_stratify[p*(i-1)+3,]))
}

beta_all_mean1=rowMeans(beta_all_1)   

plot(time,beta_all_mean1,  ylim=c(-4,2), xlab="Time", ylab=expression(beta[3](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, rep(-1,N), lty=2, col="red")

beta_low1=rep(0, nrow(beta_all_1))
beta_up1=rep(0, nrow(beta_all_1))         
for (i in 1:nrow(beta_all_1))
{
  beta_low1[i]=quantile(beta_all_1[i,], probs = 0.025)
  beta_up1[i]=quantile(beta_all_1[i,], probs = 0.975)
}
lines(time, beta_low1, lty=5)
lines(time, beta_up1, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)








############### One step Newton  ######
par(mai=c(1.1,1.1,0.5,0.4))
beta_all_2=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_2[,i]=bs8%*%(as.matrix(beta.NR.all[p*(i-1)+2,]))
}

beta_all_mean=rowMeans(beta_all_2)   
#beta_all_median=apply(beta_all_2, 1, median)

plot(time,beta_all_mean,  ylim=c(-6,5), xlab="Time", ylab=expression(beta[1](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
#plot(time,beta_all_median,  ylim=c(-2,2), xlab="Time", ylab=expression(theta[1](t)), lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, 3*sin(3*pi*time/4), lty=2, col="red")
#abline(0,0, lty=6)

beta_low=rep(0, nrow(beta_all_2))
beta_up=rep(0, nrow(beta_all_2))         
for (i in 1:nrow(beta_all_2))
{
  beta_low[i]=quantile(beta_all_2[i,], probs = 0.025)
  beta_up[i]=quantile(beta_all_2[i,], probs = 0.975)
}
lines(time, beta_low, lty=5)
lines(time, beta_up, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)

##############################################################

beta_all_1=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_1[,i]=bs8%*%(as.matrix(beta.NR.all[p*(i-1)+1,]))
}

beta_all_mean1=rowMeans(beta_all_1)   

plot(time,beta_all_mean1,  ylim=c(-2,4), xlab="Time", ylab=expression(beta[2](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, rep(1,N), lty=2, col="red")

beta_low1=rep(0, nrow(beta_all_1))
beta_up1=rep(0, nrow(beta_all_1))         
for (i in 1:nrow(beta_all_1))
{
  beta_low1[i]=quantile(beta_all_1[i,], probs = 0.025)
  beta_up1[i]=quantile(beta_all_1[i,], probs = 0.975)
}
lines(time, beta_low1, lty=5)
lines(time, beta_up1, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)





beta_all_1=matrix( rep(0, N*nloop), nrow=N)

for (i in 1:nloop)
{
  beta_all_1[,i]=bs8%*%(as.matrix(beta.NR.all[p*(i-1)+3,]))
}

beta_all_mean1=rowMeans(beta_all_1)   

plot(time,beta_all_mean1,  ylim=c(-4,2), xlab="Time", ylab=expression(beta[3](t)), main="", lty=1, cex=0.5, type='l', cex.lab=1.15, cex.axis=1.15)
lines(time, rep(-1,N), lty=2, col="red")

beta_low1=rep(0, nrow(beta_all_1))
beta_up1=rep(0, nrow(beta_all_1))         
for (i in 1:nrow(beta_all_1))
{
  beta_low1[i]=quantile(beta_all_1[i,], probs = 0.025)
  beta_up1[i]=quantile(beta_all_1[i,], probs = 0.975)
}
lines(time, beta_low1, lty=5)
lines(time, beta_up1, lty=5)
legend ("bottomleft", legend=c('Average estimate'
                               , 
                               'True effect'
                               ,
                               'CI'), yjust = 1,text.width = strwidth("1,000,000,000,000,000"),lty=c(1,2, 5), pt.cex = 0.9, cex=0.9)
